@@ -1,10 +1,12 @@
-# Copyright (c) 2005 Jens Luedicke <jensl@cpan.org>. All rights reserved.
+# Copyright (c) 2005-2006 Jens Luedicke <jensl@cpan.org>. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 
 package File::DirWalk;
+use base qw(Exporter);
 
-our $VERSION = '0.2';
+our $VERSION = '0.3';
+our @EXPORT = qw(FAILED SUCCESS ABORTED PRUNE);
 
 use strict;
 use warnings;
@@ -12,17 +14,23 @@ use warnings;
 use File::Basename;
 use File::Spec;
 
-use constant FAILED => 0;
-use constant SUCCESS => 1;
-use constant ABORTED => -1;
+use constant SUCCESS 	=> 1;
+use constant FAILED 	=> 0;
+use constant ABORTED 	=> -1;
+use constant PRUNE 	=> -10;
 
 sub new {
 	my ($class) = @_;
 	my $self = bless {}, $class;
 
-	foreach (qw(onBeginWalk onLink onFile onDirEnter onDirLeave onForEach)) {
-		$self->{$_} = sub { SUCCESS; };
-	}
+	$self->{onBeginWalk} = sub { SUCCESS };
+	$self->{onLink}      = sub { SUCCESS };
+	$self->{onFile}      = sub { SUCCESS };
+	$self->{onDirEnter}  = sub { SUCCESS };
+	$self->{onDirLeave}  = sub { SUCCESS };
+
+	$self->{depth}       = 0;
+	$self->{depth_count} = 0;
 
 	return $self;
 }
@@ -52,57 +60,68 @@ sub onDirLeave {
 	$self->{onDirLeave} = $func;
 }
 
-sub onForEach {
-	my ($self,$func) = @_;
-	$self->{onForEach} = $func;
+sub setDepth {
+	my ($self,$v) = @_;
+	$self->{depth} = $v;
+}
+
+sub getDepth {
+	my ($self) = @_;
+	return $self->{depth};
 }
 
 sub walk {
 	my ($self,$path) = @_;
 
-	if ((my $r = &{$self->{onBeginWalk}}($path)) != SUCCESS) {
+	if ((my $r = $self->{onBeginWalk}->($path)) != SUCCESS) {
 		return $r;
 	}
 
 	if (-l $path) {
 
-		if ((my $r = &{$self->{onLink}}($path)) != SUCCESS) {
+		if ((my $r = $self->{onLink}->($path)) != SUCCESS) {
 			return $r;
 		}
 
 	} elsif (-d $path) {
 
-		if ((my $r = &{$self->{onDirEnter}}($path)) != SUCCESS) {
+		if ($self->{depth} != 0) {
+			if ($self->{depth_count} == $self->{depth}) {
+				return SUCCESS;
+			}
+		}
+
+		$self->{depth_count}++;
+		if ((my $r = $self->{onDirEnter}->($path)) != SUCCESS) {
 			return $r;
 		}
 
-		my $dirh; 
-
-		opendir($dirh, $path) || return FAILED;
-
-		foreach my $f (readdir($dirh)) {
-			next if ($f eq "." or $f eq "..");
+		opendir my $dirh, $path || return FAILED;
+		my @dir_contents = readdir $dirh;
+		@dir_contents = File::Spec->no_upwards(@dir_contents);
 
+		foreach my $f (@dir_contents) {
 			# be portable.
 			my @dirs = File::Spec->splitdir($path);
 			my $path = File::Spec->catfile(@dirs, $f);
 
-			if ((my $r = &{$self->{onForEach}}($path)) != SUCCESS) {
-				return $r;
-			}
+			my $r = $self->walk($path);
 
-			if ((my $r = $self->walk($path)) != SUCCESS) {
+			if ($r == PRUNE) {
+				last;
+			} elsif ($r != SUCCESS) {
 				return $r;
 			}
 		}
 
-		closedir($dirh);
+		closedir $dirh;
 
-		if ((my $r = &{$self->{onDirLeave}}($path)) != SUCCESS) {
+		$self->{depth_count}--;
+		if ((my $r = $self->{onDirLeave}->($path)) != SUCCESS) {
 			return $r;
 		}
 	} else {
-		if ((my $r = &{$self->{onFile}}($path)) != SUCCESS) {
+		if ((my $r = $self->{onFile}->($path)) != SUCCESS) {
 			return $r;
 		}
 	}
@@ -122,13 +141,47 @@ Walk through your homedir and print out all filenames:
 
 	use File::DirWalk;
 
-	my $dw = new File::DirWalk; $dw->onFile(sub { my ($file) = @_; print
-	"$file\n";
+	my $dw = new File::DirWalk;
+	$dw->onFile(sub {
+		my ($file) = @_;
+		print "$file\n";
 
-		return File::DirWalk::SUCCESS; });
+		return File::DirWalk::SUCCESS;
+	});
 
 	$dw->walk($ENV{'HOME'});
 
+Walk through your homedir and print out all directories:
+
+	use File::DirWalk;
+
+	my $dw = new File::DirWalk;
+	$dw->onDirEnter(sub {
+		my ($dir) = @_;
+		print "$dir\n";
+
+		return File::DirWalk::SUCCESS;
+	});
+
+	$dw->walk($ENV{'HOME'});
+
+Walk through your homedir and print out all directories
+with depth 3:
+
+	use File::DirWalk;
+
+	my $dw = new File::DirWalk;
+	$dw->onDirEnter(sub {
+		my ($dir) = @_;
+		print "$dir\n";
+
+		return File::DirWalk::SUCCESS;
+	});
+
+	$dw->setDepth(3);
+	$dw->walk($ENV{'HOME'});
+
+
 =head1 DESCRIPTION
 
 This module can be used to walk through a directory tree and run own functions
@@ -173,20 +226,30 @@ to the given function when called. Function must return true.
 Specify a function to be run on each file/directory within another directory.
 The name is passed to the function when called. Function must return true.
 
+=item C<setDepth($int)>
+
+Set the directory depth:
+By default the directory depth is set to 0.
+
+=item C<getDepth>
+
+Get the directory depth;
+
 =item C<walk($path)>
 
 Begin the walk through the given directory tree. This method returns if the walk
-is finished or if one of the callbacks doesn't return true. 
+is finished or if one of the callbacks doesn't return true.
 
 =back
 
 All callback-methods expect a function reference as their argument. The
 directory- or filename  is passed to the function as the argument when called.
 The function must return true, otherwise the recursive walk is aborted and
-C<walk> returns. You don't need to define a callback if you don't need to. 
+C<walk> returns. You don't need to define a callback if you don't need to.
 
-The module provides the following constants: SUCCESS, FAILED and ABORTED (1, 0
-and -1) which you can use within your callback code. 
+The module provides the following constants: SUCCESS, FAILED, ABORTED and PRUNE (1, 0, -1, -10)
+which you can use within your callback code.
+DirWalk will stop processing the current directory if PRUNE is returned by your callback.
 
 =head1 BUGS
 
@@ -198,6 +261,8 @@ Jens Luedicke E<lt>jensl@cpan.orgE<gt> web: L<http://perldude.de/>
 
 =head1 CHANGES
 
+Version 0.3: add PRUNE constant. add option to specify the directory depth.
+
 Version 0.2: platform portability fixes and more documentation
 
 Version 0.1: first CPAN release
@@ -210,6 +275,7 @@ CPAN module.
 
 =head1 COPYRIGHT AND LICENCE
 
-Copyright (c) 2005 Jens Luedicke. All rights reserved. This program is free
+Copyright (c) 2005-2006 Jens Luedicke. All rights reserved. This program is free
 software; you can redistribute it and/or modify it under the same terms as Perl
 itself.
+
